#ifndef MSP_DATAFILE_BINARYPARSER_H_
#define MSP_DATAFILE_BINARYPARSER_H_

#include <map>
#include "parsermode.h"
#include "type.h"

namespace Msp {
namespace DataFile {

/**
Parses data in binary format.
*/
class BinaryParser: public ParserMode
{
private:
	typedef std::map<int, StatementInfo> Dictionary;
	typedef std::map<unsigned, std::string> StringMap;

	Dictionary dict;
	StringMap strings;
	unsigned float_precision = 32;
	StatementInfo *cur_info = nullptr;
	std::vector<unsigned> sub_remaining;

public:
	BinaryParser(Input &i, const std::string &s);

	Statement parse() override;
	void process_control_statement(const Statement &) override;

	const StatementKey *peek(unsigned) override;
	bool parse_and_load(unsigned, Loader &, const LoaderAction &) override;

private:
	IntType::Store parse_int();
	FloatType::Store parse_float();
	StringType::Store parse_string();
	BoolType::Store parse_bool();
	SymbolType::Store parse_symbol();
};

} // namespace DataFile
} // namespace Msp

#endif
