#ifndef COLLECTIONSOURCE_H_
#define COLLECTIONSOURCE_H_

#include <list>
#include <string>
#include <msp/io/seekable.h>

namespace Msp {
namespace DataFile {

class Collection;
class CollectionItemTypeBase;

/**
Provides automatically loadable objects for collections.  This is a base class;
see DirectorySource and PackSource for concrete classes.
*/
class CollectionSource
{
public:
	typedef std::list<std::string> NameList;

protected:
	CollectionSource() = default;
public:
	virtual ~CollectionSource() = default;

	/// Determines whether an object is available from this source.
	virtual bool is_loadable(const CollectionItemTypeBase &type, const std::string &name) const = 0;

	/** Returns the names of available objects of a specific type.  Implementors
	should use type.match_name to check which names are acceptable. */
	virtual NameList get_names(const CollectionItemTypeBase &type) const = 0;

	/// Loads an item into a collection.
	virtual void load(Collection &, const CollectionItemTypeBase &, const std::string &) const = 0;

	/** Opens a raw resource.  The caller is responsible for deleting the
	returned object when done with it. */
	virtual IO::Seekable *open(const std::string &) const = 0;
};

} // namespace DataFile
} // namespace Msp

#endif
