#include <msp/debug/demangle.h>
#include <msp/strings/format.h>
#include "except.h"

using namespace std;

namespace Msp {
namespace DataFile {

data_error::data_error(const string &s, unsigned l, const string &w):
	runtime_error(make_what(s, l, w)),
	source(s),
	line(l)
{ }

data_error::data_error(const string &s, unsigned l, const exception &e):
	runtime_error(make_what(s, l, format("%s (%s)", Debug::demangle(typeid(e).name()), e.what()))),
	source(s),
	line(l)
{ }

data_error::data_error(const string &s, unsigned l, const data_error &e):
	runtime_error(format("%s\n%s", e.what(), make_what(s, l, "Referenced here"))),
	source(s),
	line(l)
{ }

string data_error::make_what(const string &s, unsigned l, const string &w)
{
	if(l)
		return format("%s:%d: %s", s, l, w);
	else
		return format("%s: %s", s, w);
}


parse_error::parse_error(const string &t):
	runtime_error(t.empty() ? "at end of input" : format("after '%s'", t))
{ }


syntax_error::syntax_error(const string &t):
	runtime_error(t.empty() ? "at end of input" : format("at '%s'", t))
{ }


bad_definition::bad_definition(const string &w):
	runtime_error(w)
{ }


nesting_error::nesting_error(const string &w):
	logic_error(w)
{ }


unknown_keyword::unknown_keyword(const string &k):
	runtime_error(k)
{ }


invalid_signature::invalid_signature(const string &k, const string &s):
	runtime_error(format("%s %s", k, s))
{ }


no_collection::no_collection(const type_info &t):
	runtime_error(Debug::demangle(t.name()))
{ }

} // namespace DataFile
} // namespace Msp
